#include "mxcimageprovider.h"

#include <QDebug>
#include <QDir>
#include <QFileInfo>
#include <QStandardPaths>

#include "client.h"
#include "debug_out.h"

void MXCImageResponse::run() {

    QFileInfo cacheFile(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/" + m_id);

    if (m_requestedSize.isValid()) {

        cacheFile = QFileInfo(cacheFile.filePath() +
                              QString("_%1x%2").arg(m_requestedSize.width()).arg(m_requestedSize.height()));
        if (m_image.load(cacheFile.filePath())) {
            qDebug() << "Loaded cached thumbnail: " << m_id;
            emit finished();
            return;
        }

        qDebug() << "Downloading thumbnail: " << m_id;

        mtx::http::ThumbOpts opts;
        opts.mxc_url = "mxc://" + m_id.toStdString();
        opts.width = m_requestedSize.width() > 0 ? m_requestedSize.width() : -1;
        opts.height = m_requestedSize.height() > 0 ? m_requestedSize.height() : -1;
        opts.method = "scale";
        http::client().get_thumbnail(opts, [this, cacheFile](const std::string &res, mtx::http::RequestErr err) {
            if (err) {
                qDebug() << "Failed to retrieve image: " << m_id << " error: " << *err;
                m_error = "Failed download";
                emit finished();

                return;
            }

            auto data = QByteArray(res.data(), res.size());
            m_image.loadFromData(data);
            m_image = m_image.scaled(m_requestedSize, Qt::KeepAspectRatio);
            m_image.setText("mxc url", "mxc://" + m_id);

            QDir().mkpath(cacheFile.path());
            m_image.save(cacheFile.filePath(), "png");

            emit finished();
        });
    } else {
        if (m_image.load(cacheFile.filePath())) {
            qDebug() << "Loaded cached image: " << m_id;
            emit finished();
            return;
        }

        qDebug() << "Downloading image: " << m_id;

        http::client().download("mxc://" + m_id.toStdString(),
                                [this, cacheFile](const std::string &res, const std::string &content_type,
                                                  const std::string &originalFilename, mtx::http::RequestErr err) {
                                    if (err) {
                                        qDebug() << "Failed to retrieve image: " << m_id << " error: " << *err;
                                        m_error = "Failed download";
                                        emit finished();

                                        return;
                                    }

                                    auto data = QByteArray(res.data(), res.size());
                                    m_image.loadFromData(data);
                                    m_image.setText("original filename", QString::fromStdString(originalFilename));
                                    m_image.setText("mxc url", "mxc://" + m_id);

                                    QDir().mkpath(cacheFile.path());
                                    m_image.save(cacheFile.filePath(), "png");

                                    emit finished();
                                });
    }
}
